import {computed, makeObservable} from 'mobx';

import {RawSpanType, SpanChildrenLookupType, SpanType} from './types';
import {getSpanID} from './utils';

class SpanTreeModel {
  // readonly state
  span: Readonly<SpanType>;
  children: Array<SpanTreeModel> = [];

  constructor(parentSpan: SpanType, childSpans: SpanChildrenLookupType) {
    this.span = parentSpan;

    const spanID = getSpanID(parentSpan);
    const spanChildren: Array<RawSpanType> = childSpans?.[spanID] ?? [];

    // Mark descendents as being rendered. This is to address potential recursion issues due to malformed data.
    // For example if a span has a span_id that's identical to its parent_span_id.
    childSpans = {
      ...childSpans,
    };
    delete childSpans[spanID];

    this.children = spanChildren.map(span => {
      return new SpanTreeModel(span, childSpans);
    });

    makeObservable(this, {
      operationNameCounts: computed.struct,
    });
  }

  get operationNameCounts(): Map<string, number> {
    const result = new Map<string, number>();

    const operationName = this.span.op;

    if (typeof operationName === 'string' && operationName.length > 0) {
      result.set(operationName, 1);
    }

    for (const directChild of this.children) {
      const operationNameCounts = directChild.operationNameCounts;
      for (const [key, count] of operationNameCounts) {
        result.set(key, (result.get(key) ?? 0) + count);
      }
    }

    // sort alphabetically using case insensitive comparison
    return new Map(
      [...result].sort((a, b) =>
        String(a[0]).localeCompare(b[0], undefined, {sensitivity: 'base'})
      )
    );
  }
}

export default SpanTreeModel;
